@@ -580,17 +580,26 @@ gssdp_resource_browser_get_active (GSSDPResourceBrowser *resource_browser)
 static gboolean
 resource_expire (gpointer user_data)
 {
+        GSSDPResourceBrowser *resource_browser;
         Resource *resource;
+        char *usn;
 
         resource = user_data;
-        
-        g_signal_emit (resource->resource_browser,
+        resource_browser = resource->resource_browser;
+
+        /* Steal the USN pointer from the resource as we need it for the signal
+         * emission.
+         */
+        usn = resource->usn;
+        resource->usn = NULL;
+
+        g_hash_table_remove (resource->resource_browser->priv->resources, usn);
+
+        g_signal_emit (resource_browser,
                        signals[RESOURCE_UNAVAILABLE],
                        0,
-                       resource->usn);
-
-        g_hash_table_remove (resource->resource_browser->priv->resources,
-                             resource->usn);
+                       usn);
+        g_free (usn);
 
         return FALSE;
 }
@@ -771,12 +780,12 @@ resource_unavailable (GSSDPResourceBrowser *resource_browser,
         if (!g_hash_table_lookup (resource_browser->priv->resources, usn))
                 return;
 
+        g_hash_table_remove (resource_browser->priv->resources, usn);
+
         g_signal_emit (resource_browser,
                        signals[RESOURCE_UNAVAILABLE],
                        0,
                        usn);
-
-        g_hash_table_remove (resource_browser->priv->resources, usn);
 }
 
 static gboolean
